@@ -302,7 +302,7 @@ expand_format (struct obstack *obs, int argc, token_data **argv)
         }
 
       c = *fmt++;
-      if (c > sizeof ok || !ok[c])
+      if (sizeof ok <= c || !ok[c])
         {
           M4ERROR ((warning_status, 0,
                     "Warning: unrecognized specifier in `%s'", f));
